@@ -19,17 +19,15 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: Tab.cc,v 1.31 2002/08/11 21:12:10 fluxgen Exp $
+// $Id: Tab.cc,v 1.45 2002/11/27 21:51:33 fluxgen Exp $
 
 #include "Tab.hh"
 
-#ifdef HAVE_CONFIG_H
-#	include "../config.h"
-#endif // HAVE_CONFIG_H
-
 #include "i18n.hh"
 #include "DrawUtil.hh"
 #include "Screen.hh"
+#include "fluxbox.hh"
+#include "Rootmenu.hh"
 
 #include <iostream>
 using namespace std;
@@ -59,7 +57,7 @@ Tab::Tab(FluxboxWindow *win, Tab *prev, Tab *next) {
 	m_move_x = m_move_y = 0;	
 	m_prev = prev; m_next = next; 
 	m_win = win;
-	m_display = Fluxbox::instance()->getXDisplay();
+	m_display = BaseDisplay::getXDisplay();
 	
 	if ((m_win->getScreen()->getTabPlacement() == PLEFT ||
 			m_win->getScreen()->getTabPlacement() == PRIGHT) &&
@@ -121,8 +119,9 @@ void Tab::createTabWindow() {
 	Fluxbox::instance()->saveTabSearch(m_tabwin, this);	
 
 	XMapSubwindows(m_display, m_tabwin);
-
-	XMapWindow(m_display, m_tabwin);
+	// don't show if the window is iconified
+	if (!m_win->isIconic())
+		XMapWindow(m_display, m_tabwin);
 
 	decorate();
 }
@@ -192,13 +191,13 @@ void Tab::lower() {
 void Tab::loadTheme() {
 	BImageControl *image_ctrl = m_win->getScreen()->getImageControl();
 	Pixmap tmp = m_focus_pm;
-	FbTk::Texture *texture = &(m_win->getScreen()->getWindowStyle()->tab.l_focus);
+	const FbTk::Texture *texture = &(m_win->getScreen()->getWindowStyle()->tab.l_focus);
 
 	if (texture->type() & FbTk::Texture::PARENTRELATIVE ) {
-		FbTk::Texture *pt = &(m_win->getScreen()->getWindowStyle()->tab.t_focus);
-		if (pt->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
+		const FbTk::Texture &pt = m_win->getScreen()->getWindowStyle()->tab.t_focus;
+		if (pt.type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			m_focus_pm = None;
-			m_focus_pixel = pt->color().pixel();
+			m_focus_pixel = pt.color().pixel();
 		} else
 			m_focus_pm =
 				image_ctrl->renderImage(m_size_w, m_size_h, pt);
@@ -211,7 +210,7 @@ void Tab::loadTheme() {
 			m_focus_pixel = texture->color().pixel();
 		} else
 			m_focus_pm =
-				image_ctrl->renderImage(m_size_w, m_size_h, texture);
+				image_ctrl->renderImage(m_size_w, m_size_h, *texture);
 		if (tmp) image_ctrl->removeImage(tmp);
 	}
 
@@ -219,10 +218,10 @@ void Tab::loadTheme() {
 	texture = &(m_win->getScreen()->getWindowStyle()->tab.l_unfocus);
 
 	if (texture->type() & FbTk::Texture::PARENTRELATIVE ) {
-		FbTk::Texture *pt = &(m_win->getScreen()->getWindowStyle()->tab.t_unfocus);
-		if (pt->type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
+		const FbTk::Texture &pt = m_win->getScreen()->getWindowStyle()->tab.t_unfocus;
+		if (pt.type() == (FbTk::Texture::FLAT | FbTk::Texture::SOLID)) {
 			m_unfocus_pm = None;
-			m_unfocus_pixel = pt->color().pixel();
+			m_unfocus_pixel = pt.color().pixel();
 		} else
 			m_unfocus_pm =
 			image_ctrl->renderImage(m_size_w, m_size_h, pt);
@@ -232,7 +231,7 @@ void Tab::loadTheme() {
 			m_unfocus_pixel = texture->color().pixel();
 		} else
 			m_unfocus_pm =
-				image_ctrl->renderImage(m_size_w, m_size_h, texture);
+				image_ctrl->renderImage(m_size_w, m_size_h, *texture);
 	}
 	
 	if (tmp) image_ctrl->removeImage(tmp);
@@ -283,20 +282,24 @@ void Tab::withdraw() {
 void Tab::stick() {
 	Tab *tab;
  
+ 	bool wasstuck = m_win->isStuck();
+ 
 	//now do stick for all windows in the list
 	for (tab = getFirst(this); tab != 0; tab = tab->m_next) {
 		FluxboxWindow *win = tab->m_win; //just for convenience
-		if (win->isStuck()) {
+		if (wasstuck) {
 			win->blackbox_attrib.flags ^= BaseDisplay::ATTRIB_OMNIPRESENT;
 			win->blackbox_attrib.attrib ^= BaseDisplay::ATTRIB_OMNIPRESENT;
 			win->stuck = false;
-			if (!win->isIconic()) {
-				BScreen *screen = win->getScreen();
-				screen->reassociateWindow(win, screen->getCurrentWorkspace()->workspaceID(), true);
-			}
 
 		} else {
 			win->stuck = true;
+			BScreen *screen = win->getScreen();
+			if (!win->isIconic() && !(win->getWorkspaceNumber() !=
+					screen->getCurrentWorkspaceID())) {
+				screen->reassociateWindow(win, screen->getCurrentWorkspaceID(), true);
+			}
+
 			win->blackbox_attrib.flags |= BaseDisplay::ATTRIB_OMNIPRESENT;
 			win->blackbox_attrib.attrib |= BaseDisplay::ATTRIB_OMNIPRESENT;
 		}
@@ -356,45 +359,42 @@ void Tab::shade() {
 // TODO: the "draw in pressed mode" 
 //-----------------------------------
 void Tab::draw(bool pressed) const {	
-	unsigned int tabtext_w;
+	XClearWindow(m_display, m_tabwin);
+	
+	if (m_win->getTitle().size() == 0) // we don't have anything to draw
+		return;
 
 	GC gc = ((m_win->isFocused()) ? m_win->getScreen()->getWindowStyle()->tab.l_text_focus_gc :
 		m_win->getScreen()->getWindowStyle()->tab.l_text_unfocus_gc);
 
-	// Different routines for drawing rotated text
-	if ((m_win->getScreen()->getTabPlacement() == PLEFT ||
-			m_win->getScreen()->getTabPlacement() == PRIGHT) &&
-			(!m_win->isShaded() && m_win->getScreen()->isTabRotateVertical())) {
+	Theme::WindowStyle *winstyle = m_win->getScreen()->getWindowStyle();
+	size_t dlen = m_win->getTitle().size();
+	
+	size_t max_width = m_size_w; // special cases in rotated mode
+	if (winstyle->tab.font.isRotated() && !m_win->isShaded())
+		max_width = m_size_h;
 
-		tabtext_w = DrawUtil::XRotTextWidth(m_win->getScreen()->getWindowStyle()->tab.rot_font,
-						m_win->getTitle().c_str(), m_win->getTitle().size());
-		tabtext_w += (m_win->frame.bevel_w * 4);
-
-		DrawUtil::DrawRotString(m_display, m_tabwin, gc,
-				m_win->getScreen()->getWindowStyle()->tab.rot_font,
-				m_win->getScreen()->getWindowStyle()->tab.font.justify,
-				tabtext_w, m_size_w, m_size_h,
-				m_win->frame.bevel_w, m_win->getTitle().c_str());
-
-	} else {
-		if (I18n::instance()->multibyte()) { // TODO: maybe move this out from here?
-			XRectangle ink, logical;
-			XmbTextExtents(m_win->getScreen()->getWindowStyle()->tab.font.set,
-					m_win->getTitle().c_str(), m_win->getTitle().size(),
-					&ink, &logical);
-			tabtext_w = logical.width;
-		} else {
-			tabtext_w = XTextWidth(
-					m_win->getScreen()->getWindowStyle()->tab.font.fontstruct,
-					m_win->getTitle().c_str(), m_win->getTitle().size());
-		}
-		tabtext_w += (m_win->frame.bevel_w * 4);
-
-		DrawUtil::DrawString(m_display, m_tabwin, gc,
-				&m_win->getScreen()->getWindowStyle()->tab.font,
-				tabtext_w, m_size_w,
-				m_win->frame.bevel_w, m_win->getTitle().c_str());
+	int dx = DrawUtil::doAlignment(max_width, m_win->frame.bevel_w,
+		winstyle->tab.justify,
+		winstyle->tab.font,
+		m_win->getTitle().c_str(), m_win->getTitle().size(), dlen);
+	
+	int dy = winstyle->tab.font.ascent() + m_win->frame.bevel_w;
+	// swap dx and dy if we're rotated
+	if (winstyle->tab.font.isRotated() && !m_win->isShaded()) {
+		int tmp = dy;
+		dy = m_size_h - dx - 2*m_win->frame.bevel_w; // upside down (reverse direction)
+		dx = tmp;
 	}
+	
+	winstyle->tab.font.drawText(
+		m_tabwin,
+		m_win->getScreen()->getScreenNumber(),
+		gc,
+		m_win->getTitle().c_str(), dlen,
+		dx, dy,
+		!m_win->isShaded());
+
 }
 
 //-----------------------------------------------
@@ -406,10 +406,10 @@ void Tab::draw(bool pressed) const {
 int Tab::setPositionShadingHelper(bool shaded) {
 	if (shaded) {
 		return m_win->getYFrame() + m_win->getTitleHeight() + 
-				m_win->getScreen()->getBorderWidth2x();
+			m_win->getScreen()->getBorderWidth2x();
 	} else {
 		return m_win->getYFrame() + m_win->getHeight() +
-				m_win->getScreen()->getBorderWidth2x();
+			m_win->getScreen()->getBorderWidth2x();
 	}
 }
 
@@ -843,14 +843,14 @@ void Tab::motionNotifyEvent(XMotionEvent *me) {
 				case Toolbar::TOPLEFT:
 				case Toolbar::TOPCENTER:
 				case Toolbar::TOPRIGHT:
-					dtty = m_win->getScreen()->getToolbar()->getExposedHeight() +
+					dtty = m_win->getScreen()->getToolbar()->exposedHeight() +
 							m_win->getScreen()->getBorderWidth();
 					dbby = m_win->getScreen()->getHeight();
 					break;
 
 				default:
 					dtty = 0;
-					dbby = m_win->getScreen()->getToolbar()->getY();
+					dbby = m_win->getScreen()->getToolbar()->y();
 					break;
 				}
 		
@@ -936,9 +936,10 @@ void Tab::insert(Tab *tab) {
 	for (; last->m_next!=0; last=last->m_next); 
 	//do sticky before we connect it to the chain
 	//sticky bit on window
-	if (m_win->isStuck() && !tab->m_win->isStuck() ||
-			!m_win->isStuck() && tab->m_win->isStuck())
-			tab->m_win->stick(); //this will set all the m_wins in the list
+	if (m_win->isStuck() != tab->m_win->isStuck()) {
+			tab->m_win->stuck = !m_win->stuck; // it will toggle
+			tab->stick(); //this will set all the m_wins in the list
+	}
 	
 	//connect the tab to this chain
 	
