@@ -22,7 +22,7 @@
 // FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 // DEALINGS IN THE SOFTWARE.
 
-// $Id: fluxbox.cc,v 1.73 2002/08/30 14:07:38 fluxgen Exp $
+// $Id: fluxbox.cc,v 1.82 2002/11/27 22:00:19 fluxgen Exp $
 
 
 #include "fluxbox.hh"
@@ -39,9 +39,8 @@
 #include "StringUtil.hh"
 #include "Resource.hh"
 #include "XrmDatabaseHelper.hh"
-#ifdef SLIT
-#include "Slit.hh"
-#endif // SLIT
+#include "AtomHandler.hh"
+
 
 //Use GNU extensions
 #ifndef	 _GNU_SOURCE
@@ -52,6 +51,16 @@
 #include "../config.h"
 #endif // HAVE_CONFIG_H
 
+#ifdef SLIT
+#include "Slit.hh"
+#endif // SLIT
+#ifdef USE_GNOME
+#include "Gnome.hh"
+#endif // USE_GNOME
+#ifdef USE_NEWWMSPEC
+#include "Ewmh.hh"
+#endif //USE_NEWWMSPEC
+
 // X headers
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -84,28 +93,20 @@
 #include <sys/select.h>
 #endif // HAVE_SYS_SELECT_H
 
-#ifdef HAVE_SIGNAL_H
-#include <signal.h>
-#endif // HAVE_SIGNAL_H
-
-#ifdef		HAVE_SYS_SIGNAL_H
-#	include <sys/signal.h>
-#endif // HAVE_SYS_SIGNAL_H
-
 #ifdef		HAVE_SYS_STAT_H
-#	include <sys/types.h>
-#	include <sys/stat.h>
+#include <sys/types.h>
+#include <sys/stat.h>
 #endif // HAVE_SYS_STAT_H
 
 #ifdef		TIME_WITH_SYS_TIME
-#	include <sys/time.h>
-#	include <time.h>
+#include <sys/time.h>
+#include <time.h>
 #else // !TIME_WITH_SYS_TIME
-#	ifdef		HAVE_SYS_TIME_H
-#		include <sys/time.h>
-#	else // !HAVE_SYS_TIME_H
-#		include <time.h>
-#	endif // HAVE_SYS_TIME_H
+#ifdef		HAVE_SYS_TIME_H
+#include <sys/time.h>
+#else // !HAVE_SYS_TIME_H
+#include <time.h>
+#endif // HAVE_SYS_TIME_H
 #endif // TIME_WITH_SYS_TIME
 
 #ifdef		HAVE_LIBGEN_H
@@ -118,6 +119,7 @@
 #include <string>
 #include <memory>
 #include <algorithm>
+#include <typeinfo>
 
 using namespace std;
 using namespace FbTk;
@@ -319,7 +321,7 @@ key(0)
 		abort();
 	}
 	
-	//setup signals
+	//catch system signals
 	SignalHandler *sigh = SignalHandler::instance();
 	
 	sigh->registerHandler(SIGSEGV, this);
@@ -339,15 +341,24 @@ key(0)
 
 	//singleton pointer
 	singleton = this;
+
+	// setup atom handlers
+#ifdef USE_GNOME
+	m_atomhandler.push_back(new Gnome()); // for gnome 1 atom support
+#endif //USE_GNOME
+#ifdef USE_NEWWMSPEC
+	m_atomhandler.push_back(new Ewmh()); // for Extended window manager atom support
+#endif // USE_NEWWMSPEC
+
 	grab();
 	
 	setupConfigFiles();
 	
 	if (! XSupportsLocale())
-		fprintf(stderr, "X server does not support locale\n");
+		cerr<<"Warning: X server does not support locale"<<endl;
 
 	if (XSetLocaleModifiers("") == 0)
-		fprintf(stderr, "cannot set locale modifiers\n");
+		cerr<<"Warning: cannot set locale modifiers"<<endl;
 
 // Set default values to member variables
 
@@ -375,14 +386,25 @@ key(0)
 		char scrname[128], altscrname[128];
 		sprintf(scrname, "session.screen%d", i);
 		sprintf(altscrname, "session.Screen%d", i);
-		BScreen *screen = new BScreen(m_screen_rm, this, scrname, altscrname, i);
-
+		BScreen *screen = new BScreen(m_screen_rm, scrname, altscrname, i);
 		if (! screen->isScreenManaged()) {
-			delete screen;
+			delete screen;			
 			continue;
 		}
-
 		screenList.push_back(screen);
+		
+		// attach screen signals to this
+		screen->currentWorkspaceSig().attach(this);
+		screen->workspaceCountSig().attach(this);
+		screen->workspaceNamesSig().attach(this);
+		screen->clientListSig().attach(this);
+		
+		// initiate atomhandler for screen specific stuff
+		for (size_t atomh=0; atomh<m_atomhandler.size(); ++atomh) {
+			m_atomhandler[atomh]->initForScreen(*screen);
+		}
+
+		
 	}
 
 	I18n *i18n = I18n::instance();
@@ -405,13 +427,20 @@ key(0)
 	timer.fireOnce(True);
 
 	//create keybindings handler and load keys file	
-	key = new Keys(getXDisplay(), StringUtil::expandFilename(*m_rc_keyfile).c_str());
+	auto_ptr<Keys> tmp(new Keys(getXDisplay(), StringUtil::expandFilename(*m_rc_keyfile).c_str()));
+	key = tmp;
 
 	ungrab();
 }
 
 
-Fluxbox::~Fluxbox(void) {
+Fluxbox::~Fluxbox() {
+	// destroy atomhandlers
+	while (!m_atomhandler.empty()) {
+		delete m_atomhandler.back();
+		m_atomhandler.pop_back();
+	}
+	
 	std::list<MenuTimestamp *>::iterator it = menuTimestamps.begin();
 	std::list<MenuTimestamp *>::iterator it_end = menuTimestamps.end();
 	for (; it != it_end; ++it) {
@@ -423,8 +452,6 @@ Fluxbox::~Fluxbox(void) {
 		delete ts;
 	}
 	
-	delete key;
-	key = 0;
 }
 
 //---------- setupConfigFiles -----------
@@ -462,9 +489,9 @@ void Fluxbox::setupConfigFiles() {
 			createMenu = true;
 
 	} else {
-		#ifdef DEBUG
+#ifdef DEBUG
 		cerr <<__FILE__<<"("<<__LINE__<<"): Creating dir: " << dirname.c_str() << endl;
-		#endif // DEBUG
+#endif // DEBUG
 
 		// create directory with perm 700
 		if (mkdir(dirname.c_str(), 0700)) {
@@ -645,7 +672,10 @@ void Fluxbox::handleEvent(XEvent * const e) {
 			win = new FluxboxWindow(e->xmaprequest.window);
 			if (!win->isManaged()) {
 				delete win;
-				win = 0;
+				win = 0;				
+			} else {
+				// attach signals
+				attachSignals(*win);
 			}
 		}
 
@@ -884,9 +914,9 @@ void Fluxbox::handleButtonEvent(XButtonEvent &be) {
 		FluxboxWindow *win = (FluxboxWindow *) 0;
 		Basemenu *menu = (Basemenu *) 0;
 
-		#ifdef		SLIT
+#ifdef		SLIT
 		Slit *slit = (Slit *) 0;
-		#endif // SLIT
+#endif // SLIT
 
 		Toolbar *tbar = (Toolbar *) 0;			
 		Tab *tab = 0;
@@ -901,10 +931,10 @@ void Fluxbox::handleButtonEvent(XButtonEvent &be) {
 		} else if ((menu = searchMenu(be.window))) {
 			menu->buttonPressEvent(&be);
 
-		#ifdef		SLIT
+#ifdef	SLIT
 		} else if ((slit = searchSlit(be.window))) {
 			slit->buttonPressEvent(&be);
-		#endif // SLIT
+#endif // SLIT
 
 		} else if ((tbar = searchToolbar(be.window))) {
 			tbar->buttonPressEvent(&be);
@@ -1107,17 +1137,12 @@ void Fluxbox::handleClientMessage(XClientMessageEvent &ce) {
 			win->changeBlackboxHints(&net);
 		}
 	} else {
-		bool val = false;
-		#ifdef GNOME
-		val  = checkGnomeAtoms(ce);
-		#endif //!GNOME
-	
-		#ifdef NEWWMSPEC
-		if (!val)
-			val = checkNETWMAtoms(ce);
-		#endif //!NEWWMSPEC
-		//disable `unused`-warning
-		val = true;
+		FluxboxWindow *win = searchWindow(ce.window);
+		BScreen *screen = searchScreen(ce.window);
+		
+		for (size_t i=0; i<m_atomhandler.size(); ++i) {
+			m_atomhandler[i]->checkClientMessage(ce, screen, win);
+		}
 	}
 }
 //----------- handleKeyEvent ---------------
@@ -1312,9 +1337,9 @@ void Fluxbox::handleKeyEvent(XKeyEvent &ke) {
 					DisplayString(getXDisplay()));
 				sprintf(displaystring + strlen(displaystring) - 1, "%d",
 					screen->getScreenNumber());		
-				#ifdef DEBUG
+#ifdef DEBUG
 				cerr<<__FILE__<<"("<<__LINE__<<"): Executing:"<<key->getExecCommand().c_str()<<endl;
-				#endif						
+#endif // DEBUG
 						
 				bexec(key->getExecCommand().c_str(), displaystring);
 				#else
@@ -1531,86 +1556,13 @@ void Fluxbox::doWindowAction(Keys::KeyAction action, const int param) {
 	}
 
 }
-#ifdef GNOME
-//---------------- checkGnomeAtoms ---------------
-// Tries to find Gnome atoms in message
-// Returns true on success else false
-//---------------------------------------------
-bool Fluxbox::checkGnomeAtoms(XClientMessageEvent &ce) {
-	BScreen *screen = 0;
-	FluxboxWindow *win = 0;
-	win = searchWindow(ce.window);
-	screen = searchScreen(ce.window);
 
-	if (ce.message_type == getGnomeWorkspaceAtom()) {
-#ifdef DEBUG
-		cerr<<__FILE__<<"("<<__LINE__<<"): Got workspace atom="<<ce.data.l[0]<<endl;
-#endif//!DEBUG
-		if ( win !=0 && // the message sent to client window?
-				win->getScreen() && ce.data.l[0] >= 0 &&
-				ce.data.l[0] < (signed)win->getScreen()->getCount()) {
-			win->getScreen()->changeWorkspaceID(ce.data.l[0]);
-					
-		} else if (screen!=0 && //the message sent to root window?
-				ce.data.l[0] >= 0 &&
-				ce.data.l[0] < (signed)screen->getCount())
-			screen->changeWorkspaceID(ce.data.l[0]);
-		return true;
-	} else if (win) {
-		if (ce.message_type == getGnomeStateAtom()) {
-#ifdef DEBUG
-			cerr<<__FILE__<<"("<<__LINE__<<"): _WIN_STATE"<<endl;
-#endif // DEBUG
-			
-#ifdef DEBUG
-			cerr<<__FILE__<<"("<<__LINE__<<"): Mask of members to change:"<<
-				hex<<ce.data.l[0]<<dec<<endl; // mask_of_members_to_change
-			cerr<<"New members:"<<ce.data.l[1]<<endl;
-#endif // DEBUG
-	
-			//get new states			
-			int flag = ce.data.l[0] & ce.data.l[1];
-			//set states			
-			win->setGnomeState(flag);
-			
-		} else if (ce.message_type == getGnomeHintsAtom()) {
-#ifdef DEBUG
-			cerr<<__FILE__<<"("<<__LINE__<<"): _WIN_HINTS"<<endl;
-#endif // DEBUG
-
-		} else 
-			return false; //the gnome atom wasn't found or not supported
-	} else	
-		return false; //no gnome atom
-
-	return true;
-}
-#endif //!GNOME
-
-#ifdef NEWWMSPEC
-//----------- checkNETWMAtoms -------------------
-// Tries to find NEWWM atom in clientmessage
-// Returns true on success else false
-//-----------------------------------------------
-bool Fluxbox::checkNETWMAtoms(XClientMessageEvent &ce) {
-	
-	if (ce.message_type == getNETWMDesktopAtom()) { //_NET_WM_DESKTOP
-		BScreen *screen = searchScreen(ce.window);
-
-		if (screen && ce.data.l[0] >= 0 &&
-			ce.data.l[0] < screen->getCount())
-			screen->changeWorkspaceID(ce.data.l[0]);
-	} else return false;
-
-	return true;
-}
-#endif //!NEWWMSPEC
-
-void Fluxbox::handleEvent(SignalEvent * const sig) {
+// handle system signals here
+void Fluxbox::handleSignal(int signum) {
 	I18n *i18n = I18n::instance();
 	static int re_enter = 0;
 
-	switch (sig->signum) {
+	switch (signum) {
 		case SIGCHLD: // we don't want the child process to kill us
 			waitpid(-1, 0, WNOHANG | WUNTRACED);
 		break;
@@ -1636,7 +1588,7 @@ void Fluxbox::handleEvent(SignalEvent * const sig) {
 			i18n->getMessage(
 				 FBNLS::BaseDisplaySet, FBNLS::BaseDisplaySignalCaught,
 				 "%s:	signal %d caught\n"),
-				getApplicationName(), sig);
+				getApplicationName(), signum);
 
 			if (! isStartup() && ! re_enter) {
 				re_enter = 1;
@@ -1655,12 +1607,92 @@ void Fluxbox::handleEvent(SignalEvent * const sig) {
 			abort();
 		break;
 	}
-
-	
-
 }
 
 
+void Fluxbox::update(FbTk::Subject *changedsub) {
+//TODO: fix signaling, this does not look good
+	if (typeid(*changedsub) == typeid(FluxboxWindow)) {
+		FluxboxWindow::WinSubject *winsub = dynamic_cast<FluxboxWindow::WinSubject *>(changedsub);
+		FluxboxWindow &win = winsub->win();
+		if ((&(win.hintSig())) == changedsub) { // hint signal
+#ifdef DEBUG
+			cerr<<__FILE__<<"("<<__LINE__<<") WINDOW hint signal from "<<&win<<endl;
+#endif // DEBUG
+			for (size_t i=0; i<m_atomhandler.size(); ++i) {
+				if (m_atomhandler[i]->update())
+					m_atomhandler[i]->updateHints(win);
+			}
+		} else if ((&(win.stateSig())) == changedsub) { // state signal
+#ifdef DEBUG
+			cerr<<__FILE__<<"("<<__LINE__<<") WINDOW state signal from "<<&win<<endl;
+#endif // DEBUG
+			for (size_t i=0; i<m_atomhandler.size(); ++i) {
+				if (m_atomhandler[i]->update())
+					m_atomhandler[i]->updateState(win);
+			}
+		} else if ((&(win.workspaceSig())) == changedsub) {  // workspace signal
+#ifdef DEBUG
+			cerr<<__FILE__<<"("<<__LINE__<<") WINDOW workspace signal from "<<&win<<endl;
+#endif // DEBUG
+			for (size_t i=0; i<m_atomhandler.size(); ++i) {
+				if (m_atomhandler[i]->update())
+					m_atomhandler[i]->updateWorkspace(win);
+			}
+		} else {
+#ifdef DEBUG
+			cerr<<__FILE__<<"("<<__LINE__<<"): WINDOW uncought signal from "<<&win<<endl;
+#endif // DEBUG
+		}
+		
+	} else if (typeid(*changedsub) == typeid(BScreen::ScreenSubject)) {
+		BScreen::ScreenSubject *subj = dynamic_cast<BScreen::ScreenSubject *>(changedsub);
+		BScreen &screen = subj->screen();
+		if ((&(screen.workspaceCountSig())) == changedsub) {
+#ifdef DEBUG	
+			cerr<<__FILE__<<"("<<__LINE__<<"): SCREEN workspace count signal"<<endl;
+#endif // DEBUG
+			for (size_t i=0; i<m_atomhandler.size(); ++i) {
+				if (m_atomhandler[i]->update())
+					m_atomhandler[i]->updateWorkspaceCount(screen);
+			}
+		} else if ((&(screen.workspaceNamesSig())) == changedsub) {
+#ifdef DEBUG
+			cerr<<__FILE__<<"("<<__LINE__<<"): SCREEN workspace names signal"<<endl;
+#endif // DEBUG
+			for (size_t i=0; i<m_atomhandler.size(); ++i) {
+				if (m_atomhandler[i]->update())
+					m_atomhandler[i]->updateWorkspaceNames(screen);
+			}
+		} else if ((&(screen.currentWorkspaceSig())) == changedsub) {
+#ifdef DEBUG
+			cerr<<__FILE__<<"("<<__LINE__<<"): SCREEN current workspace signal"<<endl;
+#endif // DEBUG	
+			for (size_t i=0; i<m_atomhandler.size(); ++i) {
+				if (m_atomhandler[i]->update())
+					m_atomhandler[i]->updateCurrentWorkspace(screen);
+			}
+		} else if ((&(screen.clientListSig())) == changedsub) {
+#ifdef DEBUG
+			cerr<<__FILE__<<"("<<__LINE__<<"): SCREEN client list signal"<<endl;
+#endif // DEBUG
+			for (size_t i=0; i<m_atomhandler.size(); ++i) {
+				if (m_atomhandler[i]->update())
+					m_atomhandler[i]->updateClientList(screen);
+			}
+		}
+	}
+}
+
+void Fluxbox::attachSignals(FluxboxWindow &win) {
+	win.hintSig().attach(this);
+	win.stateSig().attach(this);
+	win.workspaceSig().attach(this);
+	for (size_t i=0; i<m_atomhandler.size(); ++i) {
+		m_atomhandler[i]->setupWindow(win);
+	}
+}
+
 BScreen *Fluxbox::searchScreen(Window window) {
 	BScreen *screen = (BScreen *) 0;
 	ScreenList::iterator it = screenList.begin();
@@ -1740,7 +1772,7 @@ void Fluxbox::saveTabSearch(Window window, Tab *data) {
 	tabSearch[window] = data;
 }
 
-#ifdef		SLIT
+#ifdef SLIT
 void Fluxbox::saveSlitSearch(Window window, Slit *data) {
 	slitSearch[window] = data;
 }
@@ -1756,12 +1788,10 @@ void Fluxbox::removeGroupSearch(Window window) {
 	groupSearch.erase(window);
 }
 
-
 void Fluxbox::removeMenuSearch(Window window) {
 	menuSearch.erase(window);
 }
 
-
 void Fluxbox::removeToolbarSearch(Window window) {
 	toolbarSearch.erase(window);
 }
@@ -1792,7 +1822,7 @@ void Fluxbox::restart(const char *prog) {
 }
 
 
-void Fluxbox::shutdown(void) {
+void Fluxbox::shutdown() {
 	BaseDisplay::shutdown();
 
 	XSetInputFocus(getXDisplay(), PointerRoot, None, CurrentTime);
@@ -1812,7 +1842,7 @@ void Fluxbox::shutdown(void) {
 //------ save_rc --------
 //saves resources
 //----------------------
-void Fluxbox::save_rc(void) {
+void Fluxbox::save_rc() {
 
 	XrmDatabase new_blackboxrc = 0;
 	
@@ -1998,7 +2028,7 @@ void Fluxbox::getDefaultDataFilename(char *name, string &filename) {
 	filename = string(getenv("HOME")+string("/.")+RC_PATH+string("/")+name);
 }
 
-void Fluxbox::load_rc(void) {
+void Fluxbox::load_rc() {
 	XrmDatabaseHelper database;
 	
 	//get resource filename
@@ -2006,14 +2036,14 @@ void Fluxbox::load_rc(void) {
 
 	if (dbfile.size() != 0) {
 		if (!m_resourcemanager.load(dbfile.c_str())) {
-			cerr<<"Faild to load database:"<<dbfile<<endl;
+			cerr<<"Failed to load database:"<<dbfile<<endl;
 			cerr<<"Trying with: "<<DEFAULT_INITFILE<<endl;
 			if (!m_resourcemanager.load(DEFAULT_INITFILE))
-				cerr<<"Faild to load database: "<<DEFAULT_INITFILE<<endl;
+				cerr<<"Failed to load database: "<<DEFAULT_INITFILE<<endl;
 		}
 	} else {
 		if (!m_resourcemanager.load(DEFAULT_INITFILE))
-			cerr<<"Faild to load database: "<<DEFAULT_INITFILE<<endl;
+			cerr<<"Failed to load database: "<<DEFAULT_INITFILE<<endl;
 	}
 	
 	XrmValue value;
@@ -2078,7 +2108,7 @@ void Fluxbox::load_rc(void) {
 	cerr<<__FILE__<<": Loading groups ("<<*m_rc_groupfile<<")"<<endl;
 #endif // DEBUG
 	if (!Workspace::loadGroups(*m_rc_groupfile)) {
-		cerr<<"Faild to load groupfile: "<<*m_rc_groupfile<<endl;
+		cerr<<"Failed to load groupfile: "<<*m_rc_groupfile<<endl;
 	}
 }
 
@@ -2087,14 +2117,14 @@ void Fluxbox::load_rc(BScreen *screen) {
 	string dbfile(getRcFilename());
 	if (dbfile.size() != 0) {
 		if (!m_screen_rm.load(dbfile.c_str())) {
-			cerr<<"Faild to load database:"<<dbfile<<endl;
+			cerr<<"Failed to load database:"<<dbfile<<endl;
 			cerr<<"Trying with: "<<DEFAULT_INITFILE<<endl;
 			if (!m_screen_rm.load(DEFAULT_INITFILE))
-				cerr<<"Faild to load database: "<<DEFAULT_INITFILE<<endl;
+				cerr<<"Failed to load database: "<<DEFAULT_INITFILE<<endl;
 		}
 	} else {
 		if (!m_screen_rm.load(DEFAULT_INITFILE))
-			cerr<<"Faild to load database: "<<DEFAULT_INITFILE<<endl;
+			cerr<<"Failed to load database: "<<DEFAULT_INITFILE<<endl;
 	}
 	
 	XrmDatabaseHelper database;
@@ -2406,7 +2436,7 @@ void Fluxbox::real_reconfigure() {
 //------------- reconfigureTabs ----------
 // Reconfigure all tabs size and increase steps
 // ---------------------------------------
-void Fluxbox::reconfigureTabs(void) {
+void Fluxbox::reconfigureTabs() {
 	//tab reconfiguring
 	TabList::iterator it = tabSearch.begin();
 	TabList::iterator it_end = tabSearch.end();
@@ -2427,7 +2457,7 @@ void Fluxbox::reconfigureTabs(void) {
 	}
 }
 
-void Fluxbox::checkMenu(void) {
+void Fluxbox::checkMenu() {
 	Bool reread = False;
 	std::list<MenuTimestamp *>::iterator it = menuTimestamps.begin();
 	std::list<MenuTimestamp *>::iterator it_end = menuTimestamps.end();
@@ -2445,14 +2475,14 @@ void Fluxbox::checkMenu(void) {
 }
 
 
-void Fluxbox::rereadMenu(void) {
+void Fluxbox::rereadMenu() {
 	reread_menu_wait = True;
 
 	if (! timer.isTiming()) timer.start();
 }
 
 
-void Fluxbox::real_rereadMenu(void) {
+void Fluxbox::real_rereadMenu() {
 	std::list<MenuTimestamp *>::iterator it = menuTimestamps.begin();
 	std::list<MenuTimestamp *>::iterator it_end = menuTimestamps.end();
 	for (; it != it_end; ++it) {
@@ -2498,7 +2528,7 @@ void Fluxbox::saveMenuFilename(const char *filename) {
 }
 
 
-void Fluxbox::timeout(void) {
+void Fluxbox::timeout() {
 	if (reconfigure_wait)
 		real_reconfigure();
 
@@ -2523,7 +2553,7 @@ void Fluxbox::setFocusedWindow(FluxboxWindow *win) {
 		old_wkspc = old_screen->getWorkspace(old_win->getWorkspaceNumber());
 
 		old_win->setFocusFlag(False);
-		old_wkspc->menu()->setItemSelected(old_win->getWindowNumber(), false);
+		old_wkspc->menu().setItemSelected(old_win->getWindowNumber(), false);
 		
 	}
 
@@ -2532,16 +2562,16 @@ void Fluxbox::setFocusedWindow(FluxboxWindow *win) {
 		ScreenList::iterator winscreen = 
 			std::find(screenList.begin(), screenList.end(),
 				win->getScreen());
-		if (winscreen == screenList.end())
+		if (winscreen == screenList.end()) {
 			focused_window = 0; // the window pointer wasn't valid, mark no window focused
-		else {
+		} else {
 			screen = *winscreen;
 			tbar = screen->getToolbar();
 			wkspc = screen->getWorkspace(win->getWorkspaceNumber());		
 			focused_window = win;     // update focused window
 			win->setFocusFlag(True); // set focus flag
 			// select this window in workspace menu
-			wkspc->menu()->setItemSelected(win->getWindowNumber(), true);
+			wkspc->menu().setItemSelected(win->getWindowNumber(), true);
 		}
 	} else
 		focused_window = 0;
